@@ -8,7 +8,7 @@ selection method.
 """
 
 #
-# Copyright (c) 2001 - 2015 The SCons Foundation
+# Copyright (c) 2001 - 2016 The SCons Foundation
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -30,7 +30,7 @@ selection method.
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #
 
-__revision__ = "src/engine/SCons/Platform/win32.py rel_2.3.5:3329:275e75118ad4 2015/06/20 11:18:26 bdbaddog"
+__revision__ = "src/engine/SCons/Platform/win32.py rel_2.5.1:3735:9dc6cee5c168 2016/11/03 14:02:02 bdbaddog"
 
 import os
 import os.path
@@ -60,10 +60,8 @@ except AttributeError:
 else:
     parallel_msg = None
 
-    import builtins
-
-    _builtin_file = builtins.file
-    _builtin_open = builtins.open
+    _builtin_file = file
+    _builtin_open = open
 
     class _scons_file(_builtin_file):
         def __init__(self, *args, **kw):
@@ -78,8 +76,8 @@ else:
                                       0)
         return fp
 
-    builtins.file = _scons_file
-    builtins.open = _scons_open
+    file = _scons_file
+    open = _scons_open
 
 try:
     import threading
