@@ -10,7 +10,7 @@ selection method.
 """
 
 #
-# Copyright (c) 2001-7,2010 The SCons Foundation
+# Copyright (c) 2001 - 2016 The SCons Foundation
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -852,30 +852,16 @@ def generate(env):
         )
     _detect(env)
 
-    try:
-        env.AddMethod(DocbookEpub, "DocbookEpub")
-        env.AddMethod(DocbookHtml, "DocbookHtml")
-        env.AddMethod(DocbookHtmlChunked, "DocbookHtmlChunked")
-        env.AddMethod(DocbookHtmlhelp, "DocbookHtmlhelp")
-        env.AddMethod(DocbookPdf, "DocbookPdf")
-        env.AddMethod(DocbookMan, "DocbookMan")
-        env.AddMethod(DocbookSlidesPdf, "DocbookSlidesPdf")
-        env.AddMethod(DocbookSlidesHtml, "DocbookSlidesHtml")
-        env.AddMethod(DocbookXInclude, "DocbookXInclude")
-        env.AddMethod(DocbookXslt, "DocbookXslt")
-    except AttributeError:
-        # Looks like we use a pre-0.98 version of SCons...
-        from SCons.Script.SConscript import SConsEnvironment
-        SConsEnvironment.DocbookEpub = DocbookEpub        
-        SConsEnvironment.DocbookHtml = DocbookHtml
-        SConsEnvironment.DocbookHtmlChunked = DocbookHtmlChunked
-        SConsEnvironment.DocbookHtmlhelp = DocbookHtmlhelp
-        SConsEnvironment.DocbookPdf = DocbookPdf
-        SConsEnvironment.DocbookMan = DocbookMan
-        SConsEnvironment.DocbookSlidesPdf = DocbookSlidesPdf
-        SConsEnvironment.DocbookSlidesHtml = DocbookSlidesHtml
-        SConsEnvironment.DocbookXInclude = DocbookXInclude
-        SConsEnvironment.DocbookXslt = DocbookXslt
+    env.AddMethod(DocbookEpub, "DocbookEpub")
+    env.AddMethod(DocbookHtml, "DocbookHtml")
+    env.AddMethod(DocbookHtmlChunked, "DocbookHtmlChunked")
+    env.AddMethod(DocbookHtmlhelp, "DocbookHtmlhelp")
+    env.AddMethod(DocbookPdf, "DocbookPdf")
+    env.AddMethod(DocbookMan, "DocbookMan")
+    env.AddMethod(DocbookSlidesPdf, "DocbookSlidesPdf")
+    env.AddMethod(DocbookSlidesHtml, "DocbookSlidesHtml")
+    env.AddMethod(DocbookXInclude, "DocbookXInclude")
+    env.AddMethod(DocbookXslt, "DocbookXslt")
 
 
 def exists(env):
